@@ -2,7 +2,7 @@
 # dgit
 # Debian::Dgit: functions common to dgit and its helpers and servers
 #
-# Copyright (C) 2015-2019  Ian Jackson
+# Copyright (C) 2015-2016  Ian Jackson
 #
 #    This program is free software; you can redistribute it and/or modify
 #    it under the terms of the GNU General Public License as published by
@@ -29,7 +29,7 @@ use Config;
 use Digest::SHA;
 use Data::Dumper;
 use IPC::Open2;
-use File::Path qw(:DEFAULT make_path);
+use File::Path;
 use File::Basename;
 use Dpkg::Control::Hash;
 use Debian::Dgit::ExitStatus;
@@ -51,12 +51,10 @@ BEGIN {
 		      server_branch server_ref
                       stat_exists link_ltarget rename_link_xf
 		      hashfile
-                      fail failmsg ensuredir ensurepath
-		      must_getcwd executable_on_path
+                      fail failmsg ensuredir must_getcwd executable_on_path
                       waitstatusmsg failedcmd_waitstatus
 		      failedcmd_report_cmd failedcmd
                       runcmd shell_cmd cmdoutput cmdoutput_errok
-		      @git
                       git_rev_parse changedir_git_toplevel git_cat_file
 		      git_get_ref git_get_symref git_for_each_ref
                       git_for_each_tag_referring is_fast_fwd
@@ -68,8 +66,7 @@ BEGIN {
 		      reflog_cache_insert reflog_cache_lookup
                       $package_re $component_re $suite_re $deliberately_re
 		      $distro_re $versiontag_re $series_filename_re
-		      $orig_f_comp_re $orig_f_sig_re
-		      $tarball_f_ext_re $orig_f_tail_re
+		      $orig_f_comp_re $orig_f_sig_re $orig_f_tail_re
 		      $extra_orig_namepart_re
 		      $git_null_obj
                       $branchprefix
@@ -108,16 +105,13 @@ our $series_filename_re = qr{(?:^|\.)series(?!\n)$}s;
 our $extra_orig_namepart_re = qr{[-0-9a-zA-Z]+};
 our $orig_f_comp_re = qr{orig(?:-$extra_orig_namepart_re)?};
 our $orig_f_sig_re = '\\.(?:asc|gpg|pgp)';
-our $tarball_f_ext_re = "\\.tar(?:\\.\\w+)?(?:$orig_f_sig_re)?";
-our $orig_f_tail_re = "$orig_f_comp_re$tarball_f_ext_re";
+our $orig_f_tail_re = "$orig_f_comp_re\\.tar(?:\\.\\w+)?(?:$orig_f_sig_re)?";
 our $git_null_obj = '0' x 40;
 our $ffq_refprefix = 'ffq-prev';
 our $gdrlast_refprefix = 'debrebase-last';
 our $printdebug_when_debuglevel = 1;
 our $debugcmd_when_debuglevel = 1;
 
-our (@git) = qw(git);
-
 # these three all go together, only valid after record_maindir
 our $maindir;
 our $maindir_gitdir;
@@ -224,7 +218,9 @@ sub shellquote {
 sub printcmd {
     my $fh = shift @_;
     my $intro = shift @_;
-    print $fh $intro." ".(join '', shellquote @_)."\n" or confess "$!";
+    print $fh $intro," " or confess "$!";
+    print $fh shellquote @_ or confess "$!";
+    print $fh "\n" or confess "$!";
 }
 
 sub debugcmd {
@@ -313,12 +309,6 @@ sub ensuredir ($) {
     confess "mkdir $dir: $!";
 }
 
-sub ensurepath ($$) {
-    my ($firsttocreate, $subdir) = @_; # creates necessary bits of $subidr
-    ensuredir $firsttocreate;
-    make_path "$firsttocreate/$subdir";
-}
-
 sub must_getcwd () {
     my $d = getcwd();
     defined $d or fail f_ "getcwd failed: %s\n", $!;
@@ -659,10 +649,7 @@ sub resolve_upstream_version ($$) {
 	}
 	$used = $tried[-1];
 	$message = f_ 'using upstream from git tag %s', $used;
-    } elsif ($new_upstream =~ m{^refs/tags/($versiontag_re)$}s) {
-	$message = f_ 'using upstream from git tag %s', $1;
-	$used = $1;
-    }	
+    }
     $new_upstream = git_rev_parse $new_upstream;
 
     return ($new_upstream, $used, $message);
@@ -895,7 +882,7 @@ sub reflog_cache_insert ($$$) {
     # When we no longer need to support squeeze, use --create-reflog
     # instead of this:
     my $parent = $ref; $parent =~ s{/[^/]+$}{};
-    ensurepath "$maindir_gitcommon/logs", "$parent";
+    ensuredir "$maindir_gitcommon/logs/$parent";
     my $makelogfh = new IO::File "$maindir_gitcommon/logs/$ref", '>>'
       or confess "$!";
 
@@ -989,7 +976,7 @@ sub reflog_cache_lookup ($$) {
 #    changedir playground
 #    changedir $maindir
 #
-#    playtree_setup
+#    playtree_setup $local_git_cfg
 #            # ^ call in some (perhaps trivial) subdir of playground
 #
 #    rmtree playground
@@ -1048,28 +1035,28 @@ sub fresh_playground ($) {
 
 # ----- playtrees -----
 
-sub playtree_setup () {
+sub playtree_setup (;$) {
+    my ($t_local_git_cfg) = @_;
+    $t_local_git_cfg //= $local_git_cfg;
     # for use in the playtree
     # $maindir must be set, eg by calling record_maindir or fresh_playground
+    runcmd qw(git init -q);
+    runcmd qw(git config gc.auto 0);
+    foreach my $copy (qw(user.email user.name user.useConfigOnly
+                         core.sharedRepository
+                         core.compression core.looseCompression
+                         core.bigFileThreshold core.fsyncObjectFiles)) {
+	my $v = $t_local_git_cfg->{$copy};
+	next unless $v;
+	runcmd qw(git config), $copy, $_ foreach @$v;
+    }
     # this is confusing: we have
     #   .                   playtree, not a worktree, has .git/, our cwd
     #   $maindir            might be a worktree so
     #   $maindir_gitdir     contains our main working "dgit", HEAD, etc.
     #   $maindir_gitcommon  the shared stuff, including .objects
-
-    # we need to invoke git-playtree-setup via git because
-    # there may be config options it needs which are only available
-    # to us, sensibly, in @git
-
-    # And, we look for it in @INC too.  This is a bit perverse.
-    # We do this because in the Debian packages we want to have
-    # a copy of this script in each binary package, rather than
-    # making yet another .deb or tangling the dependencies.
-    # @INC is conveniently available.
-    my $newpath = join ':', +(grep { !m/:/ } @INC),
-	          '/usr/share/dgit', $ENV{PATH};
-    runcmd qw(env), "PATH=$newpath", @git, qw(playtree-setup .);
-
+    rmtree('.git/objects');
+    symlink "$maindir_gitcommon/objects",'.git/objects' or confess "$!";
     ensuredir '.git/info';
     open GA, "> .git/info/attributes" or confess "$!";
     print GA "* $negate_harmful_gitattrs\n" or confess "$!";
